// AUTHOR: Rohan Allison
// COPYRIGHT: 2006
//  License: Artistic
// Filename: SettingsDialog.cpp

#include <wx/wx.h>
#include <wx/arrstr.h>
#include <wx/spinctrl.h>
#include <wx/statline.h>
#include <wx/colordlg.h>
#include <wx/notebook.h>
#include <wx/filepicker.h>
#include <wx/html/htmlwin.h>
// #include <wx/statline.h>

#include <wx/print.h>
// #include <wx/fdrepdlg.h>
#include <stc.h>

#include "globaldefs.h"
#include "syntax.h"
#include "FindReplDialog.h"
#include "ZionDialogs.h"
#include "zionedit.h"
#include "ModeOptsPanel.h"

#include "SettingsDialog.h"
// Now defined in syntax.h #define NUMSECTIONS 11

///Track current stylenum, lexer and syntax struct
static int s_stylenum;
static int s_currentLex;
static smallSyntax * s_currentStruct;
static wxPoint s_pos;
static int s_parentWidth;

//!SYNOPSIS: Whenever any aspect of a style is changed those changes will be temporarily stored in a TempStyle object
//! The TempStyles hashmap will be searched to see if a TempStyle object already exists for that style.
//! If one already exists, just update it. Else, create a new TempStyle object, initializing it with the current values of the style.
//! On OK, copy all TempStyles to their corresponding syntax struct and delete would be nice

TempStyles tempStyles; //! Define our hash map of pointers to class TempStyle   
//TMO tempModeOpts;

TempStyle * CheckForTempStyle( smallSyntax * aSyntax, int stylenum )
{
    wxString index = wxString::Format(_T("%s-%d"), aSyntax->name, stylenum);  //! Create unique id
    TempStyle * tptr; tptr = (TempStyle * ) NULL; wxString str;
    //! See if it already exists
    TempStyles::iterator it;  bool alreadyExist = false;
    for( it = tempStyles.begin(); it != tempStyles.end(); ++it ) {
        if( it->first == index )  { alreadyExist = true; tptr = it->second; /* retrieve it */ break;  }
    }
    return ( alreadyExist ? tptr : NULL );
}

//! A new TempStyle instance will be created on the first change to a style.
TempStyle::TempStyle( smallSyntax * aSyntax, int stylenum )
{
    m_syntax = aSyntax;  //! Save the pointer to the syntax struct (mem. location will not change)
    m_stylenum = stylenum;
}

TempStyle::~TempStyle() { }

void TempStyle::Apply()
{   
    if(m_bold)  strncpy( m_syntax->styles[m_stylenum].bold, "bold", 4 );
    else strcpy( m_syntax->styles[m_stylenum].bold, "" );
    if( m_italic ) strncpy( m_syntax->styles[m_stylenum].italic, "italic", 6 );
    else strcpy( m_syntax->styles[m_stylenum].italic, "" );
    strncpy( m_syntax->styles[m_stylenum].fore, m_fore.c_str(), 6 );
    strncpy( m_syntax->styles[m_stylenum].back, m_back.c_str(), 6 );
    strncpy( m_syntax->styles[m_stylenum].face, m_face.c_str(), 31 );
    strncpy( m_syntax->styles[m_stylenum].size, m_size.c_str(), 3 );
}

//------------------------------------------
/** SettingsDialog is the top-level dialog for this module. The parent is the edit module.
Actual style number and current lexer are passed in. From these we must also derive:
1) the address of the syntax structure corresponding to the current lexer
2) the style index corresponding to the style number. ( Remember: style numbers are not all in sequence
so a style index could be less than the actual style number ) **/
SettingsDialog::SettingsDialog( wxWindow * parent, int stylenum, int currentLex, wxPoint pos, int parentWidth )
    : wxDialog( parent, wxID_ANY, _T("Editor Settings"), pos, wxDefaultSize, wxDEFAULT_DIALOG_STYLE )
{
    ///INIT VARS
//    m_parent = (ZionEdit *)parent;
    s_pos = pos; ///Save calling position for other local dialogs
    s_parentWidth = parentWidth;
    m_cppWordsChanged = false;
    m_sqlWordsChanged = false;
    m_javaWordsChanged = false;
    m_csWordsChanged = false;
    ///Init Static Vars - visible only to this module
    s_stylenum = stylenum;
    s_currentLex = currentLex;
    s_currentStruct = syntaxStructs[ currentLex ];
    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );

    m_notebook = new wxNotebook(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxNB_MULTILINE);
    topSizer->Add( m_notebook, 0,  wxALIGN_CENTER_HORIZONTAL |wxALL, 4);
    
    //! PAGES ARE ADDED WITH M_NOTEBOOK AS PARENT NOT SettingsDialog
    ///Page order is mirrored in int pg2lex[] in syntax.cpp and hardcoded ("user words") in at least zionedit.cpp;
    ///Safer to add as last page
    m_generalOptsPanel = new GeneralOptsPanel(m_notebook, (ZionEdit *)parent);
    m_extPanel = new ExtPanel(m_notebook);
    m_notebook->AddPage( (wxNotebookPage *) m_generalOptsPanel, _T("Gobal Options") );
    m_notebook->AddPage( (wxNotebookPage *) m_extPanel, _T("Ext. Map") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &defaultSyntax), _T("Plain Text") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &cppSyntax), _T("C++") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &perlSyntax), _T("Perl") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &rubySyntax), _T("Ruby") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &htmlSyntax), _T("HTML") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &pythonSyntax), _T("Python") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &javaSyntax), _T("Java") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &csSyntax), _T("C#") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &sqlSyntax), _T("SQL") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &cssSyntax), _T("CSS") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &lispSyntax), _T("Lisp") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &f77Syntax), _T("Fortran77") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &fortranSyntax), _T("Fortran9x") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &makeSyntax), _T("MakeFile") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &batchSyntax), _T("BATCH") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &asmSyntax), _T("ASM") );
    m_notebook->AddPage( (wxNotebookPage *) new ModePanel(m_notebook, &vbscriptSyntax), _T("VB") );

    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    m_okButton = new wxButton( this, wxID_OK, _T("OK") );
    buttonSizer->Add( m_okButton,  0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    m_okButton->SetDefault();
    buttonSizer->AddSpacer(12);
    buttonSizer->Add( new wxButton(this, wxID_CANCEL, "Cancel"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);   //! wxID_CANCEL handled by the parent
    topSizer->Add(buttonSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 4);

    ///Preselect page of curr. lexer
    size_t i; bool found = false;
    for( i=0; i < NUMSECTIONS; i++) {  ///NUMSECTIONS defined in syntax.h
        if( pg2lex[i] == s_currentLex ) {
            found = true; break;
        }
    }
    if(found) m_notebook->SetSelection( i+2 ); ///Preselect page

    SetSizer(topSizer);  ///use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    topSizer->Layout();
    ///Positioning - supplied by edit module
    int dlgW = GetSize().GetWidth();
    int edgeOff = 20;
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 1;
    
    int x = pos.x + parentWidth - dlgW - edgeOff;
    //int y = pos.y + 40;
    SetSize( x, 40, wxDefaultCoord, wxDefaultCoord );  //absolute y
}

SettingsDialog::~SettingsDialog() { }

BEGIN_EVENT_TABLE( SettingsDialog, wxDialog )
    EVT_BUTTON( wxID_OK, SettingsDialog::OnOK )
    EVT_BUTTON( wxID_CANCEL, SettingsDialog::OnCancel )
//    EVT_NOTEBOOK_PAGE_CHANGED( wxID_ANY, OnPageChanged ) //handle a child event
END_EVENT_TABLE()

//void SettingsDialog::OnPageChanged( wxNotebookEvent & event )
//{
//    int old; //ModePanel * mp;
//    if( (old = event.GetOldSelection() ) == 3 ) { /// -1, 0, 1 -- invalid
//        m_cppWords2 = ( (ModePanel *) (m_notebook->GetPage( old )) )->m_words2->GetValue();
//        //Diag //mp = ;m_parent->AddText( wxString::Format(_T("C++ User Words (WordList2): %s "), mp->m_words2->GetValue()) );
//    }
//}

void SettingsDialog::OnOK( wxCommandEvent &event )
{
    TempStyles::iterator it;
//    //! DIAG
//     wxString str( _T("List of Temp Styles \n") );  //TempStyles::iterator it;
//    for( it = tempStyles.begin(); it != tempStyles.end(); ++it ) {
//        str.Append( it->first + _T(" -> ") + it->second->m_syntax->styles[it->second->m_stylenum].item + "\n" );
//        str.Append( _T("Bold: ") ); str.Append( it->second->m_bold? _T("yes") : _T("no") );
//        str.Append( _T(", Italic: ") ); str.Append( it->second->m_italic? _T("yes") : _T("no") );
//        str.Append( _T(", Fore: ") + it->second->m_fore );
//        str.Append( _T(", Back: ") + it->second->m_back );
//        str.Append( _T(", Face: ") + it->second->m_face );
//        str.Append( _T(", Size: ") + it->second->m_size + "\n" );
//    }
//    wxMessageDialog tstyles(this, str, _T("Temp Styles"), wxOK );
//    tstyles.ShowModal();
//     //! END DIAG

   //! Apply our TempStyles
//    TempStyles::iterator it;
    for( it = tempStyles.begin(); it != tempStyles.end(); ++it ) {
        it->second->Apply();
//        if( it->second) {
//            it->second->Apply();
//            delete it->second;  it->second = NULL; //! Delete the tempstyle
//        }
//        tempStyles.erase(it->first);  //! Delete the entry in the hash
    }
    tempStyles.clear(); ///Remove all elements - Releasing memory (I hope)?
    
    
    //! Apply our TempModeOpts
    TMO::iterator itmo;
    for( itmo = tempModeOpts.begin(); itmo != tempModeOpts.end(); ++itmo ) {
            itmo->second->Apply();
//        if( itmo->second ) {
//            itmo->second->Apply();
//            delete itmo->second; itmo->second = NULL;
//        }
//        tempModeOpts.erase(itmo->first);  //! Delete the entry in the hash
    }
	tempModeOpts.clear();
    ///Load cppWords2 ///NOTEBOOK PAGE SPECIFIC CODE!
    ModePanel * mp = (ModePanel *) (m_notebook->GetPage(3));
    if( mp != NULL && mp->m_showWords->IsChecked() ) {
        m_cppWordsChanged = true;  ///Assume words changed
         m_cppWords2 = mp->m_words2->GetValue();
    }
    ///Load javaWords2 ///Java is PAGE 8
    mp = (ModePanel *) (m_notebook->GetPage(8));
    if( mp != NULL && mp->m_showWords->IsChecked() ) {
        m_javaWordsChanged = true;  ///Assume words changed
        m_javaWords2 = mp->m_words2->GetValue();
    }
    ///Load csWords2 ///C# is PAGE 9
    mp = (ModePanel *) (m_notebook->GetPage(9));
    if( mp != NULL && mp->m_showWords->IsChecked() ) {
        m_csWordsChanged = true;  ///Assume words changed
        m_csWords2 = mp->m_words2->GetValue();
    }
    ///Load sqlWords2 ///C# is PAGE 9
    mp = (ModePanel *) (m_notebook->GetPage(10));
    if( mp != NULL && mp->m_showWords->IsChecked() ) {
        m_sqlWordsChanged = true;  ///Assume words changed
        m_sqlWords2 = mp->m_words2->GetValue();
    }
    
    event.Skip();  //! Allow the default handler to do it's thing
}

void SettingsDialog::OnCancel( wxCommandEvent &event)
{
    //TempStyles::iterator it;
    //for( it = tempStyles.begin(); it != tempStyles.end(); ++it ) {
    //    if( it->second) {
    //        delete it->second;  it->second = NULL; //! Delete the tempstyle
    //    }
    //    tempStyles.erase(it->first);  //! Delete the entry in the hash
    //}
	tempStyles.clear();
    //TMO::iterator itmo;
    //for( itmo = tempModeOpts.begin(); itmo != tempModeOpts.end(); ++itmo ) {
    //    if( itmo->second ) {
    //        delete itmo->second; itmo->second = NULL;  //! Delete the tempModeOpt
    //    }
    //    tempModeOpts.erase(itmo->first);  //! Delete the entry in the hash
    //}
	tempModeOpts.clear();

    event.Skip();  //! Allow the default handler to do it's thing
}

//--------------------------------------------------------------------------------------MODE PANEL
ModePanel::ModePanel( wxWindow * parent, smallSyntax * aSyntax )
    : wxPanel(parent)
{
    m_Syntax = aSyntax; //! Save the mode's structure for later ref.
    
    m_modeSizer = new wxBoxSizer( wxHORIZONTAL );
    ///RETRIEVE STYLE NUM
    /** Style numbers do not necessarily correspond to style indexes of the styles array **/
    int stylenum =-1;  ///All our check are against -1 not zero !!
    int styleIdx = 0;
    ///Check that max stylenum not exceeded    
    if( s_stylenum <= aSyntax->styles[aSyntax->stylesUsed - 1].styleNum ) ///Last index is # of styles used -1 !!!
        stylenum= s_stylenum;
        
    bool textMode = (aSyntax == &defaultSyntax); ///Text mode uses the default syntax struct
    //! Scan for the index where style number = styleByte
    for( int i=0; i < aSyntax->stylesUsed; i++ ) { 
        if( stylenum == aSyntax->styles[i].styleNum ) { styleIdx = i ;  break; }
    } //! Could do some additional checks here for safety
    
    ///LEFT SIDE
    ///SETUP LIST BOX
    wxBoxSizer* m_leftSizer = new wxBoxSizer( wxVERTICAL );
    wxString str; wxArrayString Elements;
    for( int i = 0; i < aSyntax->stylesUsed; i++ ) {
        str = aSyntax->styles[i].item; Elements.Add(str);
    }
    wxListBox * stylesList = new wxListBox( this, ZION_ELEMENTSLISTBOX,
            wxDefaultPosition, wxDefaultSize, Elements, wxLB_HSCROLL );
    str.Printf( _T("%s General Options"), aSyntax->name );
    stylesList->InsertItems( 1, &str, 0 ); /// #of items, string, pos
    str = _T("----------------------------------");
    stylesList->InsertItems( 1, &str, 1 ); /// #of items, string, pos
    ///Determine selection
    if( m_Syntax == s_currentStruct && stylenum != -1 ) stylesList->SetSelection(styleIdx +2); 
    else stylesList->SetSelection(0);
    bool notgAC = wxString(_T("C++")) != m_Syntax->name  && wxString(_T("JAVA")) != m_Syntax->name
            && wxString(_T("C#")) != m_Syntax->name && wxString(_T("SQL")) != m_Syntax->name ;
    if( notgAC ) stylesList->SetMinSize( wxSize(186, 344) );
    else stylesList->SetMinSize( wxSize(186, 274) );
    m_leftSizer->Add( stylesList, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 3 );
    
    ///SETUP USER WORDLIST
    m_showWords = new wxCheckBox( this, ZION_SHOWWORDS, _T("Edit Global AutoComp Words") ); ///Checkbox  is the trigger to show the words: m_words2
    m_words2 = new wxTextCtrl( this, wxID_ANY, wxString::Format( _T("%s"), _T("") ),
            wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE );
    m_words2->SetMinSize(wxSize(186, 50));  /// -1 == don't care //! Size must be requested BEFORE adding to the parent sizer
    m_words2->Enable(false);
    
    m_leftSizer->Add( m_showWords, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 3 );
    m_leftSizer->Add( m_words2, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 3 );
    if( notgAC ) {
        m_leftSizer->Show( m_showWords, false ); ///Don't even show the trigger
        m_leftSizer->Show( m_words2, false ); ///Always hidden at first for performance reasons
    }
    m_leftSizer->AddSpacer(4);

    m_modeSizer->Add( m_leftSizer, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
    if( textMode ) { ///Text mode won't show the left side at all
        m_modeSizer->Show( m_leftSizer, false, true );  /// false = don't show, true = recursive ///make the whole sizer disappear
        m_modeSizer->AddSpacer(100); ///Text Mode only
    }
    
    ///RIGHT SIDE
    m_stylePanel = (StylePanel * ) NULL; m_modeOptsPanel = (ModeOptsPanel *) NULL;
    /** Critical Code **/
    if( m_Syntax == s_currentStruct && stylenum != -1 && ! textMode ) {
        m_stylePanel = new StylePanel( this, m_Syntax, styleIdx );
        m_modeSizer->Add( m_stylePanel, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
    }
    else {
        m_modeOptsPanel = new ModeOptsPanel( this, m_Syntax, s_pos, s_parentWidth );
        m_modeSizer->Add( m_modeOptsPanel, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
    }
    SetSizer(m_modeSizer);
    m_modeSizer->SetSizeHints( this );  //! Coordinate minimal window size
    m_modeSizer->Layout();
}

ModePanel::~ModePanel() { }

BEGIN_EVENT_TABLE(ModePanel, wxPanel)
    EVT_LISTBOX( ZION_ELEMENTSLISTBOX, ModePanel::OnLBSelect )
    EVT_CHECKBOX( ZION_SHOWWORDS, ModePanel::OnShowWords )
END_EVENT_TABLE()

void ModePanel::OnShowWords( wxCommandEvent & event )
{
    if( m_showWords->IsChecked() ) {
        m_words2->Enable(true);
        m_Syntax->lexer == wxSTC_LEX_SQL ? m_words2->AppendText( m_Syntax->words6 )
            : m_words2->AppendText( m_Syntax->words2 );
        //m_leftSizer->Show( m_words2, true);  ///true = show, true = recursive
//        m_leftSizer->SetSizeHints(this);  //! Coordinate minimal window size
//        m_leftSizer->Layout();
        //m_modeSizer->Layout();
    }
    else {
        m_words2->Clear();
        m_words2->Enable(false);
    }
}

void ModePanel::OnLBSelect( wxCommandEvent & event )
{
    int sel = 0;  sel = event.GetSelection();
    if( sel == 1 )  return;  /// Separator is inactive
    //! ModeOptsPanel  going to  StylePanel
    if( sel != 0 && m_modeOptsPanel != NULL ) { //&& m_rightPanel->GetId() == ZION_MODE_OPTS )   //! Checking window IDs to determine ModeOptPanel or StylePanel
        m_modeSizer->Detach( (wxWindow *) m_modeOptsPanel );
        m_modeOptsPanel->Destroy(); m_modeOptsPanel = (ModeOptsPanel *) NULL;
        m_stylePanel  = new StylePanel( this, m_Syntax, sel - 2 ); //! ListBox order is critical!
        m_modeSizer->Add( m_stylePanel, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
        m_modeSizer->Layout();
    }
    /// StylePanel  going to  ModeOptsPanel
    else if( sel == 0 && m_stylePanel != NULL ) { //&& m_rightPanel->GetId() == ZION_STYLE ) 
        m_modeSizer->Detach( (wxWindow *) m_stylePanel );
        m_stylePanel->Destroy(); m_stylePanel = (StylePanel *) NULL;
        m_modeOptsPanel  = new ModeOptsPanel( this, m_Syntax, s_pos, s_parentWidth );
        m_modeSizer->Add( m_modeOptsPanel , 1, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
        m_modeSizer->Layout();
    }
    /// StylePanel going to StylePanel
    else if( sel > 0  && m_stylePanel != NULL ) {
        //StylePanel * sPanel = ( StylePanel *) m_rightPanel;
        m_stylePanel->m_stylenum = sel -2;
        m_stylePanel->SPLoadWidgets();
    }
}

//STYLE DIALOG (Just a wrapper for a stylePanel) -------------------------------------------------------------
StyleDialog::StyleDialog( wxWindow * parent, smallSyntax * aSyntax, int element, wxPoint pos, int parentWidth )
: wxDialog( parent, wxID_ANY, _T("Style Dialog"), pos, wxDefaultSize, wxDEFAULT_DIALOG_STYLE )
{
    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    m_stylePanel = new StylePanel( this, aSyntax, element );
    topSizer->Add( m_stylePanel, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 2);

    wxBoxSizer *buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    wxButton * okButton = new wxButton( this, wxID_OK, _T("OK") );
    buttonSizer->Add( okButton,  0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    okButton->SetDefault();
    buttonSizer->Add( new wxButton(this, wxID_CANCEL, "Cancel"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );   //! wxID_CANCEL handled by the parent
    topSizer->Add(buttonSizer,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 4);
    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    ///Positioning - supplied by edit module
    int dlgW = GetSize().GetWidth();
    int edgeOff = 50;
    if( parentWidth - dlgW > dlgW )  edgeOff = ( parentWidth - dlgW ) >> 1;
    
    int x = pos.x + parentWidth - dlgW - edgeOff;
    int y = pos.y + 60;
    SetSize( x, y, wxDefaultCoord, wxDefaultCoord );
}

BEGIN_EVENT_TABLE(StyleDialog, wxDialog)
    EVT_BUTTON( wxID_OK,  StyleDialog::OnOk)
    EVT_BUTTON( wxID_CANCEL,  StyleDialog::OnCancel)
END_EVENT_TABLE()

void StyleDialog::OnOk(wxCommandEvent & event)
{
    m_stylePanel->SaveState();
    event.Skip();
}

void StyleDialog::OnCancel(wxCommandEvent & event)
{
    m_stylePanel->Abort();
    event.Skip();
}

//STYLE PANEL------------------------------------------------------------------------------------------------------------
StylePanel::StylePanel( wxWindow * parent, smallSyntax * aSyntax, int element )
        : wxPanel( parent, ZION_STYLE ) //! ID to distinguish a Style Panel from a Mode Opts Panel
{
    m_syntax = aSyntax;  m_stylenum = element;
    /*! Create widgets */
    wxBoxSizer *topSizer = new wxBoxSizer( wxVERTICAL );
    wxString str = m_syntax->name; str.Append(_T("   ")); str.Append(m_syntax->styles[m_stylenum].item ); 
    m_staticText = new wxStaticText(this, wxID_ANY, str);
    topSizer->Add( m_staticText, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 2);
    m_foreButton = new wxButton( this, ZION_STYLE_FORE, "", wxDefaultPosition, wxDefaultSize, wxNO_BORDER);
    m_foreButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );    //! Use our colours not system's.
    m_backButton = new wxButton( this, ZION_STYLE_BACK, "", wxDefaultPosition,
            wxDefaultSize, wxNO_BORDER );
    m_backButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    
    wxBoxSizer *topHalf = new wxBoxSizer( wxHORIZONTAL);
    /*! Top Left */
    wxBoxSizer * topHalfLeft = new wxBoxSizer(wxVERTICAL);
    topHalfLeft->Add( new wxStaticText(this, wxID_ANY, "  Foreground"), 0, wxALIGN_LEFT|wxALIGN_BOTTOM|wxALL, 2 );
    topHalfLeft->Add(m_foreButton, 0, wxALIGN_LEFT|wxALIGN_TOP|wxALL, 2);

    m_bold = new wxCheckBox(this, ZION_STYLE_BOLD, "Bold");  //! We need a unique ID if we want to handle events from a widget
    m_italic = new wxCheckBox(this, ZION_STYLE_ITALIC, "Italic");
    //wxBoxSizer *checkSizer = new wxBoxSizer( wxHORIZONTAL );
    topHalfLeft->Add( m_bold,  0, wxALIGN_LEFT|wxALL, 5 );
    topHalfLeft->Add( m_italic,  0, wxALIGN_LEFT|wxALL, 5 );

    topHalf->Add(topHalfLeft,  1, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    
    /*! Top Right */
    wxStaticBoxSizer * topHalfRight = new wxStaticBoxSizer(wxVERTICAL, this, "");
    wxBoxSizer * tHR1 = new wxBoxSizer(wxHORIZONTAL);
    tHR1->Add( new wxStaticText(this, wxID_ANY, "Background"), 0, wxALIGN_LEFT|wxALL, 5);
    tHR1->Add( m_backButton, 0, wxALIGN_LEFT|wxALL, 5 );
    topHalfRight->Add( tHR1, 0, wxALIGN_LEFT|wxALL, 5);
    topHalfRight->Add(  new wxCheckBox(this, ZION_STYLE_USEPAGE, "Use Page Background",
            wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT), 0, wxALIGN_LEFT|wxALL, 5 );

    topHalf->Add(topHalfRight,  2, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    topSizer->Add( topHalf, 0,  wxALIGN_CENTER_HORIZONTAL|wxALL, 2);

    wxBoxSizer * mid = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * midL = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer * midR = new wxBoxSizer(wxVERTICAL);

    midL->Add( new wxStaticText(this, wxID_ANY, " Font Face"), 0, wxALIGN_LEFT|wxALL, 2);
    wxString fface;
    m_fontCombo = new wxComboBox( this, ZION_STYLE_FACE, fface,
         wxDefaultPosition, wxDefaultSize, g_faceNames, wxCB_READONLY | wxCB_SORT );
    midL->Add(m_fontCombo, 0, wxALIGN_LEFT|wxALL, 2);
    mid->Add(midL, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
    
    midR->Add( new wxStaticText(this, wxID_ANY, "Font Size "), 0, wxALIGN_LEFT|wxALL, 2 );
    wxString fsizes[] = {_T("8"), _T("9"),_T("10"),_T("11"),_T("12"),_T("13"),_T("14") };
    wxString startSize;
    m_spin = new wxComboBox( this, ZION_STYLE_SIZE, startSize, wxDefaultPosition,
            wxDefaultSize, 7, fsizes, wxCB_READONLY );
    midR->Add(m_spin, 0, wxALIGN_LEFT|wxALL, 2);
    mid->Add(midR, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5 );
    topSizer->Add(mid,  0, wxALIGN_CENTER_HORIZONTAL |wxALL, 2);

    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
    //m_applyButton->SetDefault();  //! activated with <enter>
    
    SPLoadWidgets(); //! Initialize widgets
}

void StylePanel::SPLoadWidgets( )
{
    m_index = wxString::Format(_T("%s-%d"), m_syntax->name, m_stylenum);  //! Create unique id
    
    TempStyle * tptr; wxString str;
    //! See if it already exists
    TempStyles::iterator it;  bool alreadyExist = false;
    for( it = tempStyles.begin(); it != tempStyles.end(); ++it ) {
        if( it->first == m_index )  { alreadyExist = true; tptr = it->second; /* save it */ break;  }
    }
    if(alreadyExist)  {
        //! Init via our tempStyle */
        //! Must set button backgnd plus our local color variable
        m_foreButton->SetBackgroundColour( m_foreColor = StringToColor(tptr->m_fore) );
        m_backButton->SetBackgroundColour( m_backColor = StringToColor(tptr->m_back) );
        m_bold->SetValue(tptr->m_bold);
        m_italic->SetValue(tptr->m_italic);
        m_fontCombo->SetValue(tptr->m_face);
        m_spin->SetValue(tptr->m_size);
    }
    else {  //! Init via syntax struct */
        m_foreButton->SetBackgroundColour( m_foreColor = StringToColor(m_syntax->styles[m_stylenum].fore) );
        m_backButton->SetBackgroundColour( m_backColor = StringToColor(m_syntax->styles[m_stylenum].back) );
        m_bold->SetValue( strcmp( m_syntax->styles[m_stylenum].bold, "bold" ) ==0 );
        m_italic->SetValue( strcmp( m_syntax->styles[m_stylenum].italic, "italic" ) ==0 );
        str = m_syntax->styles[m_stylenum].face;
        m_fontCombo->SetValue(str);
        str =  m_syntax->styles[m_stylenum].size;
        if( str == "" )  str = _T("9");  //! Correct for no value assigned
        m_spin->SetValue(str);
    }
    //! Set up our Static Text
    str = m_syntax->name; str.Append( "   "); str.Append( m_syntax->styles[m_stylenum].item );
    m_staticText->SetLabel(str);
}

StylePanel::~StylePanel() { }

BEGIN_EVENT_TABLE(StylePanel, wxPanel)
    EVT_BUTTON( ZION_STYLE_FORE,  StylePanel::OnChooseColour)
    EVT_BUTTON( ZION_STYLE_BACK,  StylePanel::OnChooseColour)
    EVT_CHECKBOX( ZION_STYLE_USEPAGE,  StylePanel::OnUsePage)
    EVT_CHECKBOX( ZION_STYLE_BOLD,  StylePanel::OnWidgetChange)
    EVT_CHECKBOX( ZION_STYLE_ITALIC,  StylePanel::OnWidgetChange)
    EVT_COMBOBOX( ZION_STYLE_FACE,  StylePanel::OnWidgetChange)
    EVT_COMBOBOX( ZION_STYLE_SIZE,  StylePanel::OnWidgetChange)
END_EVENT_TABLE()

void StylePanel::Abort() ///Cancel
{
    //! See if it already exists
    TempStyles::iterator it;
    for( it = tempStyles.begin(); it != tempStyles.end(); ++it ) {
        if( it->first == m_index ) { 
            if(it->second) { ///if we have a valid pointer
                //we want to delete without applying. it->second->Apply();
                delete it->second;  it->second = NULL; //! Delete the tempstyle
            }
            tempStyles.erase(it->first);  //! Delete the entry in the hash
        }
        break;
    }
}

void StylePanel::SaveState()
{
/*     ///If this is the line number style we want to update/create a Default TempModeOpts with the line number colors -
    ///We will still create a tempStyle for the line number style - colors will only be actually applied from the mode though.
 */
    TempStyle * tptr; ///TODO -optimize - Use CheckForTempStyle ?
    //! See if it already exists
    TempStyles::iterator it;  bool alreadyExist = false;
    for( it = tempStyles.begin(); it != tempStyles.end(); ++it ) {
        if( it->first == m_index )  { alreadyExist = true; tptr = it->second; /* save it */ break;  }
    }
    if(! alreadyExist)  {
        tptr = new TempStyle( m_syntax, m_stylenum );
        tempStyles[m_index] = tptr;  //! Register the pointer to our tempStyle into the hashmap
    }
    /*! Init or update our tempStyle */
    tptr->m_bold = m_bold->IsChecked();
    tptr->m_italic = m_italic->IsChecked();
    tptr->m_fore.Printf("%02x%02x%02x", m_foreColor.Red(),  //! Fore
            m_foreColor.Green(),  m_foreColor.Blue() );
    tptr->m_back.Printf("%02x%02x%02x", m_backColor.Red(),  //! Back
            m_backColor.Green(),  m_backColor.Blue() );
    tptr->m_face = m_fontCombo->GetValue(); //! face
    tptr->m_size = m_spin->GetValue(); //! size
    
//    //! DIAG
//     wxString str( _T("List of Temp Styles \n") );  //TempStyles::iterator it;
//    for( it = tempStyles.begin(); it != tempStyles.end(); ++it ) {
//        str.Append( it->first + _T(" -> ") + it->second->m_syntax->styles[it->second->m_stylenum].item + "\n" );
//        str.Append( _T("Bold: ") ); str.Append( it->second->m_bold? _T("yes") : _T("no") );
//        str.Append( _T(", Italic: ") ); str.Append( it->second->m_italic? _T("yes") : _T("no") );
//        str.Append( _T(", Fore: ") + it->second->m_fore );
//        str.Append( _T(", Back: ") + it->second->m_back );
//        str.Append( _T(", Face: ") + it->second->m_face );
//        str.Append( _T(", Size: ") + it->second->m_size + "\n" );
//    }
//    wxMessageDialog tstyles(this, str, _T("Temp Styles"), wxOK );
//    tstyles.ShowModal();
//     //! END DIAG
}

void StylePanel::OnWidgetChange( wxCommandEvent & event ) { SaveState(); }

void StylePanel::OnUsePage( wxCommandEvent & event )
{
    TempModeOpts * tmos;
    //! See if it already exists
    TMO::iterator it;  bool alreadyExist = false;
    for( it = tempModeOpts.begin(); it != tempModeOpts.end(); ++it ) {
        if( it->first == m_syntax->name )  { alreadyExist = true; tmos = it->second; /* save it */ break;  }
    }
    if(alreadyExist) {
        m_backButton->SetBackgroundColour( m_backColor = tmos->m_pageColor );
    }
    else m_backButton->SetBackgroundColour( m_backColor = m_syntax->pageColor );
    SaveState();
}

void StylePanel::OnChooseColour(wxCommandEvent & event)
{
    wxColourData data; int i;
    data.SetChooseFull(true);
    for (i = 0; i < 16; i++)  data.SetCustomColour(i, customColors[i]);
    if( event.GetId() == ZION_STYLE_FORE ) data.SetColour(m_foreColor);
    if( event.GetId() == ZION_STYLE_BACK ) data.SetColour(m_backColor);
    wxColourDialog dialog(this, &data);
    if (dialog.ShowModal() == wxID_OK)
    {
        wxColourData retData = dialog.GetColourData();
        wxColour col = retData.GetColour();
        for( i=0; i < 16; i++)  customColors[i] = retData.GetCustomColour(i);

        if( event.GetId() == ZION_STYLE_FORE ) {
            m_foreColor = col;  m_foreButton->SetBackgroundColour(m_foreColor);
        }
        if( event.GetId() == ZION_STYLE_BACK ) {  
            m_backColor = col;  m_backButton->SetBackgroundColour(m_backColor);
        }
        SaveState();
    }
}

//--------------------------------------------------------------------------------------
ExtPanel::ExtPanel( wxWindow * parent)   ///Extensions
    : wxPanel(parent)
{
    ///Make a working copy of the ext map. This allows us to cancel
    stringToInt::iterator it;
    for( it = ext2lex.begin(); it != ext2lex.end(); ++it ) { 
        m_exts.Add(it->first); ///load our wxArrayString
        m_e2lcopy[it->first] = it->second;  ///copy keys and values
    }
    m_combo = new wxComboBox( this, ZION_EXTCOMBO, _T(""), wxDefaultPosition,
            wxDefaultSize, m_exts, wxCB_SORT );
    /// Modes
    wxString str; wxArrayString modes;
    for( int j = 0; j < NUMSECTIONS ;  j++ )  { 
        str = smallSyntaxes[j]->name;  ///wxString is necessary here for conversion
        if( str == _T("DEFAULT") ) str = _T("Plain Text"); ///name change
        modes.Add(str);
    }
    modes.Add(_T("UNMAP") );
    m_list = new wxListBox( this, ZION_MODESLISTBOX,
            wxDefaultPosition, wxSize( -1, 250 ), modes ); //stylesList->Set(Elements);
    m_msg = new wxStaticText( this, ZION_EXTMSG, _T("") );
    
    ///Layout
    wxBoxSizer * topSizer = new wxBoxSizer(wxVERTICAL);
    topSizer->Add( new wxStaticText(this, wxID_ANY, _T("File Extension to Syntax Mode Mapping") ),
            0, wxALIGN_TOP|wxALIGN_CENTER_HORIZONTAL|wxALL, 3);

    topSizer->Add( new wxStaticText(this, wxID_ANY,
            _T("Select an extension on the left to see its mode mapping on the right.\n\
To add a new extension type it in (lowercase) then click the desired mode on the right.")),
            0, wxALIGN_TOP|wxALIGN_CENTER_HORIZONTAL|wxALL, 3);
    
    wxBoxSizer * m_extSizer = new wxBoxSizer(wxHORIZONTAL);
    m_extSizer->Add( new wxStaticText(this, wxID_ANY, _T("Extension")), 0, wxALIGN_TOP|wxALL, 3);
    m_extSizer->Add( m_combo , 0, wxALIGN_TOP|wxALL, 3);
    m_extSizer->Add( new wxStaticText(this, wxID_ANY, "Syntax Mode"), 0, wxALIGN_TOP|wxALL, 3);
    m_extSizer->Add( m_list, 0, wxALIGN_TOP|wxALL, 3);
    topSizer->Add(m_extSizer, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 3);
    
    topSizer->Add(m_msg, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 4);
    SetSizer(topSizer);
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
}

ExtPanel::~ExtPanel() { }

BEGIN_EVENT_TABLE(ExtPanel, wxPanel)
    EVT_COMBOBOX( ZION_EXTCOMBO, ExtPanel::OnComboSelect )
    EVT_LISTBOX( ZION_MODESLISTBOX, ExtPanel::OnLBSelect )
END_EVENT_TABLE()

void ExtPanel::OnLBSelect( wxCommandEvent &event) ///mode clicked so create/update mapping
{   /// listbox created from smallSyntaxes //int a = m_list->GetSelection();
    wxString str = m_combo->GetValue();
    if( str == wxEmptyString ) return;
    if( m_list->GetStringSelection() == _T("UNMAP") ) {
        if( m_e2lcopy.find(str) != m_e2lcopy.end() ) {
            m_e2lcopy.erase( str );  ///Remove entry by key
            m_msg->SetLabel(_T("Extension \"") + str + _T("\" has been Unmapped") );
        }
        else {
            m_msg->SetLabel(_T("Extension \"") + str + _T("\" not found in Extension Map!") );
        }
        return;
    }
    ///DO IT
    m_e2lcopy[ str /* m_combo->GetValue() */ ] = smallSyntaxes[m_list->GetSelection()]->lexer; ///update the hashmap. m_e2lcopy is a string keyed with extensions, values are lexer numbers
    ///Let the user know we must not change str as it is used later
    wxString str2(_T("DEFAULT"));
    m_msg->SetLabel( wxString::Format(_T("Mapped  %s  to:   %s"), ( str=="no extension" ? str: "*." + str ) /* param one */,
            (str2==smallSyntaxes[m_list->GetSelection()]->name ? _T("Plain Text") : smallSyntaxes[m_list->GetSelection()]->name) /*param two */ ) );
    
    /// ensure our combo includes any newly entered extensions
    if( m_combo->FindString(str) == wxNOT_FOUND ) {
        m_combo->Append(str);
    }
}

void ExtPanel::OnComboSelect(wxCommandEvent &event) /// an ext clicked so just show the mapping in the modes list
{
    int a = m_e2lcopy[m_combo->GetValue()]; ///Get lexer number
    m_list->SetStringSelection(lexerNameFromNum[a]); ///Point to the corresponding syntax mode
}

//GENERAL OPTS PANEL---------------------------------------------------------------------------------------------
GeneralOptsPanel::GeneralOptsPanel( wxWindow * parent, ZionEdit * zedit /* Parent of SettingsDialog ie the edit module */ ) //wxStyledTextCtrl *
    : wxPanel( parent )
{
    m_eolComboChanged = false;
    m_zedit = zedit;
    m_caretColor = g_caretColor;

    ///INIT WIDGETS
    
    m_caretButton = new wxButton( this, ZION_STYLE_CARET, _T(" | "), wxDefaultPosition,
            wxDefaultSize, wxNO_BORDER );
    m_caretButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    m_caretButton->SetBackgroundColour( wxColour(0x60, 0x60, 0x60) );
    m_caretButton->SetForegroundColour( m_caretColor );

    m_braceHiliteButton = new wxButton( this, ZION_STYLE_BRACEHILITE, _T("(   )") ); //, wxDefaultPosition, wxDefaultSize, wxNO_BORDER );
    m_braceHiliteButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    m_braceHiliteButton->SetForegroundColour(StringToColor(defaultSyntax.styles[2].fore) );
    m_braceHiliteButton->SetBackgroundColour( StringToColor(defaultSyntax.styles[2].back) );
    
    m_braceBadButton = new wxButton( this, ZION_STYLE_BRACEBAD, _T(" ) ") ); //, wxDefaultPosition, wxDefaultSize, wxNO_BORDER );
    m_braceBadButton->SetBackgroundStyle( wxBG_STYLE_COLOUR );
    m_braceBadButton->SetForegroundColour(StringToColor(defaultSyntax.styles[3].fore) );
    m_braceBadButton->SetBackgroundColour( StringToColor(defaultSyntax.styles[3].back) );
    
    wxString eols[] = {_T("Windows"), _T("Mac"),_T("Unix") };
    wxString start;
    int a =zedit->GetEolDefault(); ///Roh's mod to stc
    // (( ZionEdit *) zedit )->m_ourFrame->SetStatusText(wxString::Format(_T("a = %d"), a), 1); ///Diag
    switch(a) {  
        case wxSTC_EOL_CRLF:    start = _T("Windows"); break;
        case wxSTC_EOL_CR:       start = _T("Mac");  break;
        case wxSTC_EOL_LF:        start = _T("Unix");  break;
    }
    m_eolMode = new wxComboBox( this, ZION_EOL_MODE, start, wxDefaultPosition,
            wxDefaultSize, 3, eols, wxCB_READONLY );
            
    ///DEFAULT LEXER            
    wxString str; wxArrayString modes;
    for( int j = 0; j < NUMSECTIONS ;  j++ ) { 
        str = smallSyntaxes[j]->name;  ///wxString is necessary here for conversion
        if( str == _T("DEFAULT") ) str = _T("Plain Text"); ///name change
        modes.Add(str);
    }
    start = lexerNameFromNum[ zedit->m_startLex ];
    m_startLex = new wxComboBox( this, ZION_MODESLISTBOX, start,
            wxDefaultPosition, wxDefaultSize, modes, wxCB_READONLY );

    ///PRINT COLOUR MODE
    wxString pcols[] = {_T("Invert Colors"), _T("All Backgrounds to white"), _T("Default Background to white"), _T("Black on white"), _T("As on screen") };
    a = zedit->GetPrintColourMode();
    switch(a) {  
        case wxSTC_PRINT_NORMAL:    start = _T("As on screen"); break;
        case wxSTC_PRINT_INVERTLIGHT:       start = _T("Invert Colors");  break;
        case wxSTC_PRINT_BLACKONWHITE:        start = _T("Black on white");  break;
        case wxSTC_PRINT_COLOURONWHITE:        start = _T("All Backgrounds to white");  break;
        case wxSTC_PRINT_COLOURONWHITEDEFAULTBG:        start = _T("Default Background to white");  break;
    }
    m_pcolMode = new wxComboBox( this, wxID_ANY, start, wxDefaultPosition,
            wxDefaultSize, 5, pcols, wxCB_READONLY );
            
    ///RECENTFILES
    wxString recentFiles[] = {  _T("5"), _T("6"), _T("7"), _T("8"), _T("9"), _T("10"), _T("11"), _T("12"), _T("13"),
            _T("14"), _T("15"), _T("16"), _T("17"), _T("18"), _T("19"), _T("20"), _T("21"), _T("22"), _T("23"), _T("24")};
    start = wxString::Format( _T("%d"), g_numTopRecentFiles ); /// global var known via zionedit.h
    m_recentFiles = new wxComboBox( this, wxID_ANY, start, wxDefaultPosition,
            wxDefaultSize, 20, recentFiles, wxCB_READONLY); /// 14 = number of items
            
    ///RECENTDIRS
    wxString recentDirs[] = { _T("5"), _T("6"), _T("7"), _T("8"), _T("9"), _T("10"), _T("11"), _T("12"), _T("13"),
            _T("14"), _T("15"), _T("16"), _T("17"), _T("18"), _T("19"), _T("20"), _T("21"), _T("22"), _T("23"), _T("24"),
            _T("25"), _T("26"), _T("27"), _T("28"), _T("29"), _T("30"), _T("31"), _T("32")};
    start = wxString::Format( _T("%d"), g_numRecentDirs ); /// global var known via zionedit.h
    m_recentDirs = new wxComboBox( this, wxID_ANY, start, wxDefaultPosition,
            wxDefaultSize, 28, recentDirs, wxCB_READONLY); /// 19 = number of items
            
    ///INITIAL ZOOM LEVEL
    wxString zooms[] = { _T("+5"), _T("+4"), _T("+3"), _T("+2"), _T("+1"), _T("0"), _T("-1"), _T("-2"), _T("-3"),
            _T("-4"), _T("-5") };
    start = g_initialZoom;
    m_zoom = new wxComboBox( this, wxID_ANY, start, wxDefaultPosition,
            wxDefaultSize, 11, zooms, wxCB_READONLY ); /// 11= number of items
            
    ///PRINT LINE NUMS?
    m_dontPrintNums = new wxCheckBox(this, wxID_ANY, _T("Don't print line numbers"),
            wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    m_dontPrintNums->SetValue( ((ZionEdit *) zedit)->m_dontPrintNums );
 
    ///LINE NUM WIDTH
    wxString lineNumWidth[] = { _T("1"), _T("2"), _T("3"), _T("4"), _T("5"), _T("6") };
    start.Printf(_T("%d"), g_lineNumWidth );
    m_lineNumWidth = new wxComboBox( this, wxID_ANY, start, wxDefaultPosition,
            wxDefaultSize, 6, lineNumWidth, wxCB_READONLY ); /// 5= number of items

 
    ///PATH TO BROWSER
    m_pathToBrowser = new wxTextCtrl( this, wxID_ANY, g_pathToBrowser,  wxDefaultPosition, wxSize(280, -1) );
    m_browse = new wxButton( this, ZION_BROWSE_BROWSER, "[...]", wxDefaultPosition, wxSize(28, -1) );
    ///PATH TO API
    m_apiUseDef = new wxButton( this, ZION_API_USEDEF, _T("Use Default"), wxDefaultPosition, wxSize(62, -1) );
    m_pathToApi = new wxTextCtrl( this, wxID_ANY, g_pathToApi,  wxDefaultPosition, wxSize(282, -1) );
    m_browseApi = new wxButton( this, ZION_BROWSE_API, "...", wxDefaultPosition, wxSize(28, -1) );
    m_browseApi->SetToolTip(_T("Browse for Folder") );
    //m_apiDirPicker = new wxDirPickerCtrl( this, wxID_ANY, g_pathToApi, _T("Select Folder for API Files"), wxDefaultPosition, wxSize(300, -1) );
    
    /// CALL TIPS
    m_ctCheckBox = new wxCheckBox( this, wxID_ANY, _T("Call Tips") );
    m_ctCheckBox->SetValue( g_ctActive );
    /// AUTOCOMPLETE
    m_acCheckBox = new wxCheckBox( this, wxID_ANY, _T("Auto Complete" ),
            wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    m_acCheckBox->SetValue( g_acActive );
    wxString numChars[] = { _T("2"), _T("3"), _T("4"), _T("5"), _T("6"), _T("7") };
    start.Printf(_T("%d"), g_numACChars );
    m_numACChars = new wxComboBox( this, wxID_ANY, start, wxDefaultPosition,
            wxDefaultSize, 6, numChars, wxCB_READONLY ); /// 6= number of items
            
    /// AUTOBOOKMARKS        
    m_autoBook = new wxCheckBox( this, wxID_ANY, _T("Auto Bookmarks") );
    m_autoBook->SetValue( g_autoBook );
    
    ///AUTOZOOM
    m_autoZoom = new wxCheckBox( this, wxID_ANY, _T("Auto Zoom") );
    m_autoZoom->SetValue( g_autoZoom );
    
    ///DEFAULT Startup Size and Positioning
    m_rbNorm = new wxRadioButton( this, ZION_OPTS_STARTNORM, _T("Normal (1/2 Width, Full Height)"), wxDefaultPosition,
            wxDefaultSize, wxRB_GROUP);  ///Start a new RadioBox group
    m_rbMax = new wxRadioButton( this, ZION_OPTS_STARTMAX, _T("Maximized"), wxDefaultPosition, wxDefaultSize);
    m_rbCustom = new wxRadioButton( this, ZION_OPTS_STARTCUSTOM, _T("Custom"), wxDefaultPosition, wxDefaultSize);
    if( zedit->m_startOpt == 0 )  m_rbNorm->SetValue( true );
    if( zedit->m_startOpt == 1 )  m_rbMax->SetValue( true );
    if( zedit->m_startOpt == 2 )  m_rbCustom->SetValue( true );

    wxSize useSize( 38, -1 ); /// w, h 
    str = wxString::Format(_T("%d"), zedit->m_startPos.y );
    m_top = new wxTextCtrl( this, wxID_ANY, str );
    m_top->SetMinSize( useSize );
    str = wxString::Format(_T("%d"), zedit->m_startPos.x );
    m_left = new wxTextCtrl( this, wxID_ANY, str );
    m_left->SetMinSize( useSize );
    str = wxString::Format(_T("%d"), zedit->m_startSize.GetWidth() );
    m_width = new wxTextCtrl( this, wxID_ANY, str );
    m_width->SetMinSize( useSize );
    str = wxString::Format(_T("%d"), zedit->m_startSize.GetHeight() );
    m_height = new wxTextCtrl( this, wxID_ANY, str );
    m_height->SetMinSize( useSize );
    m_useCurrent = new wxCheckBox(this, ZION_USECURRENT, _T("Use Current Pos. and Size")/*,wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT*/ );

    ///LAYOUT
    
    wxBoxSizer * topSizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer * semiTopSizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer * edefL = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer * edefR = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer * edef1 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef2 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef3 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef4 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef5 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef6 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef7 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef8 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef9 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef10 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef11 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef12 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer * edef13 = new wxBoxSizer(wxHORIZONTAL);

    edef4->Add( new wxStaticText(this, wxID_ANY, "# of Recent Files"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef4->Add( m_recentFiles,  0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef5->Add( new wxStaticText(this, wxID_ANY, "# of Recent Dirs"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef5->Add( m_recentDirs,  0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef8->Add( new wxStaticText(this, wxID_ANY, "Startup Lexer"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef8->Add( m_startLex, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    edef11->Add( new wxStaticText(this, wxID_ANY, "Print Color Mode"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef11->Add( m_pcolMode,  0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef12->Add( m_dontPrintNums, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    edef12->AddSpacer(6);
    edef12->Add( new wxStaticText(this, wxID_ANY, "Line Number Width"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef12->Add( m_lineNumWidth, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    edefL->Add( edef4,  0, wxALIGN_RIGHT|wxALL, 3 );
    edefL->Add( edef5,  0, wxALIGN_RIGHT|wxALL, 3 );
    edefL->Add( edef8,  0, wxALIGN_RIGHT|wxALL, 3 );
    edefL->Add( edef11, 0, wxALIGN_RIGHT|wxALL, 3 );
    edefL->Add( edef12, 0, wxALIGN_RIGHT|wxALL, 3 );
    semiTopSizer->Add(edefL, 0, wxALIGN_CENTER_VERTICAL|wxALL, 3);

    edef1->Add( new wxStaticText(this, wxID_ANY, "Initial Zoom Level"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef1->Add( m_zoom, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    edef6->Add( new wxStaticText(this, wxID_ANY, "Brace Match Style"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef6->Add( m_braceHiliteButton,  0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef7->Add( new wxStaticText(this, wxID_ANY, "Brace Mismatch Style"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef7->Add( m_braceBadButton,  0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef2->Add( new wxStaticText(this, wxID_ANY, "Caret Color"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef2->Add( m_caretButton, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    edef10->Add( new wxStaticText(this, wxID_ANY, "Default End of Line"), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef10->Add( m_eolMode,  0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edefR->Add( edef1,  0, wxALIGN_RIGHT|wxALL, 3 );
    edefR->Add( edef10, 0, wxALIGN_RIGHT|wxALL, 3 );
    edefR->Add( edef2,  0, wxALIGN_RIGHT|wxALL, 3 );
    edefR->Add( edef6,  0, wxALIGN_RIGHT|wxALL, 3 );
    edefR->Add( edef7,  0, wxALIGN_RIGHT|wxALL, 3 );
    semiTopSizer->Add(edefR, 0, wxALIGN_CENTER_VERTICAL|wxALL, 3);

    topSizer->Add( semiTopSizer, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 3 );
    
    ///Lower Section
    edef9->Add( new wxStaticText(this, wxID_ANY, "Path to Web Browser"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef9->Add( m_pathToBrowser,  0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef9->Add( m_browse,  0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    topSizer->Add( edef9, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 3 );
    edef13->Add( new wxStaticText(this, wxID_ANY, "Path to API Files "), 0, wxALIGN_CENTER_VERTICAL|wxALL, 2 );
    edef13->Add( m_pathToApi,  0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef13->AddSpacer(1);
    edef13->Add( m_apiUseDef, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef13->Add( m_browseApi,  0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    topSizer->Add( edef13, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 2 );
    
    edef3->Add( m_autoBook,  0, wxALIGN_CENTER_VERTICAL|wxALL, 3);
    edef3->Add( new wxStaticText(this, wxID_ANY, "|"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 4);
    edef3->Add( m_autoZoom,  0, wxALIGN_CENTER_VERTICAL|wxALL, 3);
    edef3->Add( new wxStaticText(this, wxID_ANY, "|"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 4);
    edef3->Add( m_ctCheckBox,  0, wxALIGN_CENTER_VERTICAL|wxALL, 3);
    edef3->Add( new wxStaticText(this, wxID_ANY, "|"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 4);
    edef3->Add( m_acCheckBox, 0, wxALIGN_CENTER_VERTICAL|wxALL, 2);
    edef3->AddSpacer(3);
    edef3->Add( new wxStaticText(this, wxID_ANY, "After"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 1);
    edef3->Add( m_numACChars,  0, wxALIGN_CENTER_VERTICAL|wxALL, 1);
    edef3->Add( new wxStaticText(this, wxID_ANY, "Chars"), 0, wxALIGN_CENTER_VERTICAL|wxALL, 1);
    
    topSizer->Add( edef3, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 3 );
    
    wxStaticBoxSizer * stBox = new wxStaticBoxSizer( wxVERTICAL, this, _T("Default Startup Size and Positioning") );
    stBox->Add( m_rbNorm, 0, wxALIGN_LEFT| wxALL, 2 );
    stBox->Add( m_rbMax, 0, wxALIGN_LEFT| wxALL, 2 );
        wxBoxSizer * custSizer = new wxBoxSizer(wxHORIZONTAL);
        custSizer->Add( m_rbCustom, 0, wxALIGN_LEFT| wxALL, 0 );
        custSizer->AddSpacer(6);
        custSizer->Add( new wxStaticText( this, wxID_ANY, _T("Top") ), 0, wxALIGN_RIGHT| wxALL, 1 );
        custSizer->Add( m_top, 0, wxALIGN_RIGHT| wxALL, 1 );
        custSizer->AddSpacer(3);
        custSizer->Add( new wxStaticText( this, wxID_ANY, _T("Left") ), 0, wxALIGN_RIGHT| wxALL, 1 );
        custSizer->Add( m_left, 0, wxALIGN_RIGHT| wxALL, 1 );
        custSizer->AddSpacer(3);
        custSizer->Add( new wxStaticText( this, wxID_ANY, _T("Width") ), 0, wxALIGN_RIGHT| wxALL, 1 );
        custSizer->Add( m_width, 0, wxALIGN_RIGHT| wxALL, 1 );
        custSizer->AddSpacer(3);
        custSizer->Add( new wxStaticText( this, wxID_ANY, _T("Height") ), 0, wxALIGN_RIGHT| wxALL, 1 );
        custSizer->Add( m_height, 0, wxALIGN_RIGHT| wxALL, 1 );
        custSizer->AddSpacer(4);
        custSizer->Add( m_useCurrent, 0, wxALIGN_RIGHT| wxALL, 1 );
    stBox->Add( custSizer, 0, wxALIGN_LEFT| wxALL, 2 );

    topSizer->Add( stBox, 0, wxALIGN_CENTER_HORIZONTAL| wxALL, 3 );
    topSizer->AddSpacer(12);
    
    SetSizer(topSizer);  //use the sizer for layout
    topSizer->SetSizeHints(this);  //! Coordinate minimal window size
}

GeneralOptsPanel::~GeneralOptsPanel() { }

BEGIN_EVENT_TABLE(GeneralOptsPanel, wxPanel)
    EVT_BUTTON( ZION_BROWSE_BROWSER,  GeneralOptsPanel::OnBrowserBrowse )
    EVT_BUTTON( ZION_BROWSE_API,  GeneralOptsPanel::OnApiBrowse )
    EVT_BUTTON( ZION_API_USEDEF,  GeneralOptsPanel::OnApiUseDef )
    EVT_BUTTON( ZION_STYLE_CARET,  GeneralOptsPanel::OnChooseColor )
    EVT_BUTTON( ZION_STYLE_BRACEHILITE,  GeneralOptsPanel::OnBraceHilite )
    EVT_BUTTON( ZION_STYLE_BRACEBAD,  GeneralOptsPanel::OnBraceBad )
    EVT_COMBOBOX( ZION_EOL_MODE,  GeneralOptsPanel::OnEOLCombo )
    EVT_CHECKBOX( ZION_USECURRENT,  GeneralOptsPanel::OnUseCurrent )
END_EVENT_TABLE()

void GeneralOptsPanel::OnApiUseDef(wxCommandEvent & event) {
    m_pathToApi->SetValue( m_zedit->GetIniDir() + _T("\\API_Files") );
//    m_pathToApi->Enable( ! m_apiUseIni->IsChecked() );
//    m_browseApi->Enable( ! m_apiUseIni->IsChecked() );
}

void GeneralOptsPanel::OnUseCurrent(wxCommandEvent & event)
{
    if( m_useCurrent->IsChecked() ) {
        int width =0, height=0, xpos=0, ypos=0;
        wxFrame * frame = (wxFrame *) m_zedit->m_ourFrame;
        frame->GetSize( &width, &height );   frame->GetPosition( &xpos, &ypos );
        //frame->SetStatusText( wxString::Format(_T("Width: %d, Height: %d, xpos: %d, ypos: %d"), width, height, xpos, ypos ) );
        m_top->SetValue( wxString::Format(_T("%d"), ypos) );
        m_left->SetValue( wxString::Format(_T("%d"), xpos) );
        m_width->SetValue( wxString::Format(_T("%d"), width ) );
        m_height->SetValue( wxString::Format(_T("%d"), height ) );
    }
}

void GeneralOptsPanel::OnBrowserBrowse(wxCommandEvent & event)
{
     wxFileDialog * ofDialog = new wxFileDialog( this, "Locate Browser", 
            g_pathToBrowser.BeforeLast('\\'),
            g_pathToBrowser.AfterLast('\\'),
            "EXE files (*.exe)|*.exe|"
            "All files|*.*|"
            , wxOPEN );
            
    if( ofDialog->ShowModal() == wxID_OK) {
        m_pathToBrowser->SetValue( ofDialog->GetPath() ); }
}

void GeneralOptsPanel::OnApiBrowse(wxCommandEvent & event)
{
     wxDirDialog * ofDialog = new wxDirDialog( this, "Location of API Files", g_pathToApi,
            wxDD_DEFAULT_STYLE | wxDD_DIR_MUST_EXIST );
            
    if( ofDialog->ShowModal() == wxID_OK ) {
        m_pathToApi->SetValue( ofDialog->GetPath() ); }
}

void GeneralOptsPanel::OnEOLCombo(wxCommandEvent & event)
{
    m_eolComboChanged = true;
}

void GeneralOptsPanel::OnChooseColor(wxCommandEvent & event)
{
    wxColourData data; int i;
    data.SetChooseFull(true);
    for (i = 0; i < 16; i++)  data.SetCustomColour(i, customColors[i]);
    
    if( event.GetId() == ZION_STYLE_CARET ) data.SetColour(m_caretColor);
//    else if( event.GetId() == ZION_STYLE_INDENT ) data.SetColour(m_indentColor);

    wxColourDialog dialog(this, &data);
    if (dialog.ShowModal() == wxID_OK)
    {
        wxColourData retData = dialog.GetColourData();
        wxColour col = retData.GetColour();
        for( i=0; i < 16; i++)  customColors[i] = retData.GetCustomColour(i);

        if( event.GetId() == ZION_STYLE_CARET ) {
            m_caretColor = col;  m_caretButton->SetForegroundColour(m_caretColor);
        }
//        else if( event.GetId() == ZION_STYLE_INDENT ) {
//            m_indentColor = col;  m_indentButton->SetBackgroundColour(m_indentColor);
//        }
    }
}

void GeneralOptsPanel::OnBraceHilite(wxCommandEvent & event)
{
    StyleDialog stydia( this, &defaultSyntax, 2, s_pos, s_parentWidth );
    ///Override color inits of style dialog
    stydia.m_stylePanel->m_foreButton->SetBackgroundColour( m_braceHiliteButton->GetForegroundColour() );
    stydia.m_stylePanel->m_foreColor = m_braceHiliteButton->GetForegroundColour();
    stydia.m_stylePanel->m_backButton->SetBackgroundColour( m_braceHiliteButton->GetBackgroundColour() );
    stydia.m_stylePanel->m_backColor = m_braceHiliteButton->GetBackgroundColour();
    if( (stydia.ShowModal()) == wxID_OK ) {
        m_braceHiliteButton->SetForegroundColour( stydia.m_stylePanel->m_foreColor );
        m_braceHiliteButton->SetBackgroundColour( stydia.m_stylePanel->m_backColor );
    }
}

void GeneralOptsPanel::OnBraceBad(wxCommandEvent & event)
{
    StyleDialog stydia(this, &defaultSyntax, 3, s_pos, s_parentWidth );
    stydia.m_stylePanel->m_foreButton->SetBackgroundColour( m_braceBadButton->GetForegroundColour() );
    stydia.m_stylePanel->m_foreColor = m_braceBadButton->GetForegroundColour();
    stydia.m_stylePanel->m_backButton->SetBackgroundColour( m_braceBadButton->GetBackgroundColour() );
    stydia.m_stylePanel->m_backColor = m_braceBadButton->GetBackgroundColour();
    if( (stydia.ShowModal()) == wxID_OK )
        m_braceBadButton->SetForegroundColour( stydia.m_stylePanel->m_foreColor );
        m_braceBadButton->SetBackgroundColour( stydia.m_stylePanel->m_backColor );
}
//
//void GeneralOptsPanel::OnPage( wxCommandEvent & event)
//{
//    StyleDialog stydia( this, &defaultSyntax, 0 );
//    if( (stydia.ShowModal()) == wxID_OK )
//        m_pageButton->SetBackgroundColour( stydia.m_stylePanel->m_backColor );
//}
